#include "Sensors.h" 

Sensors sensors;

Sensors::Sensors(){
  
  for (char idx=0;idx<SENS_COUNT;idx++){
    s[idx].rxID= 0;
    s[idx].temperature= 0.0;
    s[idx].humidity= 0;
    s[idx].gracetime= 0;
    s[idx].batteryLow= 0;
    s[idx].updated= 0;
    s[idx].isValid= 0;     
    s[idx].errors= 0;
  }
  
}

void Sensors::addSensor( char idx, char Name[], char rxID ){
        
  if (idx >= SENS_COUNT){
    return;
  }
  if (idx < 0){
    return;
  }
  
  for (char i=0;i<20;i++){
        s[idx].name[i]= Name[i]; 
  }
  
  s[idx].rxID= rxID;
  s[idx].isValid= 1;
  
  
}

Sensors::sSensor * Sensors::getSensor( char idx ){
  
  if (idx >= SENS_COUNT){
    return 0;
  }
  if (idx < 0){
    return 0;
  }
  
  return &(s[idx]);
}

void Sensors::process(){
        
  static unsigned long startTime= 0;
  unsigned long currentTime= millis();
  
  // check if 1min has passed
  if ((currentTime-startTime) > 60000){
        startTime= currentTime;
    
    // check all sensors if they were updated correctly
    for (char idx=0;idx<SENS_COUNT;idx++){
      if (s[idx].isValid==0){
        continue;
      }
      
        // sensor was not updated correctly
      if (s[idx].updated==0){
        // reset gracetime
        s[idx].gracetime= 0;
        // increase error count
        if (s[idx].errors <= 99){
          s[idx].errors++;
        }
        
        
      } else {
        // sensor was updated correctly  
        s[idx].gracetime++;
        
        // clear one error every 24hours 
        if (s[idx].gracetime >= 1440 ){ //[24hours]
          s[idx].gracetime= 0;
          // so we can check once a day for errors
          if (s[idx].errors > 0){
                s[idx].errors--; 
          }
        }
      }
      
      // reset update
      s[idx].updated= 0;
    }
    
    print();
  }
  
}


void Sensors::HandleSensorData( char ID, float temperature, char humidity, char batteryLow ){
  
  for (char i=0;i<SENS_COUNT;i++){
    if (sensors.s[i].rxID == ID ){
                sensors.s[i].temperature= temperature;
      sensors.s[i].humidity= humidity;
      sensors.s[i].batteryLow= batteryLow;
      sensors.s[i].updated= 1;
      
      break;
    }
  }
    
}

void Sensors::print(){

  Serial1.println("---");
  for (char i=0;i<SENS_COUNT;i++){
    if (s[i].isValid){
        Serial1.print( s[i].name );
      Serial1.print( " " );
      Serial1.print( s[i].temperature );
      Serial1.print( " " );
      Serial1.print( s[i].humidity, DEC );
      Serial1.print( " " );
      Serial1.print( s[i].batteryLow, DEC );
      Serial1.print( " " );
      Serial1.print( s[i].errors, DEC );
      Serial1.println();
      
    }
  }  
  
}

 
